# SynergyFinder
#
# Functions in this page:
# PlotSensitiveSynergy: Plot Sensitive-Synergy plot for all the combinations in
#                       the input data

#' Plot Sensitive-Synergy Plot for All the Combinations in the Input Data
#' 
#' This function will generate a scatter plot for all the combinations in the
#' input data. The x-axis is the Combination Sensitive score (CSS).
#'
#' @param data A list object generated by function \code{\link{ReshapeData}}.
#' @param plot_synergy A character value. It indicates the synergy score for
#'  visualization. The available values are: "ZIP", "HSA", Bliss", "Leowe".
#' @param point_color An R color value. It indicates the color for the points.
#' @param point_label_color An R color value. It indicates the color for the
#' label of data points.
#'
#' @return A ggplot object.
#'
#' @author
#' \itemize{
#'   \item Shuyu Zheng \email{shuyu.zheng@helsinki.fi}
#'   \item Jing Tang \email{jing.tang@helsinki.fi}
#' }
#' 
#' @export
#'
#' @examples
#' data("mathews_screening_data")
#' data <- ReshapeData(mathews_screening_data)
#' data <- CalculateSynergy(data, method = c("ZIP"))
#' data <- CalculateSensitivity(data)
#' PlotSensitiveSynergy(data, plot_synergy = "ZIP")
PlotSensitiveSynergy <- function(data,
                                 plot_synergy,
                                 point_color = "#2D72AD",
                                 point_label_color = "#2D72AD"){
  plot_table <- data$drug_pairs
  # 1. Check the input data
  # Data structure of 'data'
  if (!is.list(data)) {
    stop("Input data is not in list format!")
  }
  if (!all(c("drug_pairs", "response") %in% names(data))) {
    stop("Input data should contain at least tow elements: 'drug_pairs' and 
         'response'. Please prepare your data with 'ReshapeData' function.")
  }
  # Parameter 'plot_synergy'
  
  avail_value <- colnames(plot_table)[endsWith(colnames(plot_table),"_synergy")]
  avail_value <- c(avail_value, sub("_synergy", "", avail_value))
  if (!plot_synergy %in% avail_value) {
    stop("The parameter 'plot_synergy = ", plot_synergy, "' is not available.",
         "Avaliable values are '", paste(avail_value, collapse = ", "),
         "'. Alternativly calculate the synergy scores with function",
         " 'CalculateSynergy'.")
  }
  if (!grepl("_synergy", plot_synergy)) {
    plot_synergy <- paste0(plot_synergy, "_synergy")
  }
  # Check CSS score in data 
  if (!"css" %in% colnames(plot_table)) {
    stop("There is no Combination Sensitivity Score (CSS) in input data. ",
         "Please run function 'CalculateSensitivity' to calculate it.")
  }
  
  # Plot title
  plot_title <- paste0(
    sub("_synergy", "", plot_synergy),
    " Synergy - Combination Sensitivity for all combinations")
  
  p <- plot_table %>%
    tidyr::unite("label", block_id, dplyr::starts_with("drug"), sep = "-") %>% 
    dplyr::select(synergy = !!plot_synergy, css, label) %>% 
    ggplot2::ggplot(mapping = aes(x = css, y = synergy)) +
    ggplot2::geom_point(colour = point_color) +
    ggrepel::geom_text_repel(
      aes(label = label),
      colour = point_label_color,
      point.padding = 0.25,
      max.overlaps = 30,
      size = .Pt2mm(10),
      fontface="bold",
      show.legend=FALSE
    ) +
    labs(
      title = plot_title,
      x = "Combination Sensitivity Score",
      y = paste0(sub("_synergy", "", plot_synergy), " Synergy Score")
    ) +
    theme(
      panel.background = ggplot2::element_rect(
        fill = "white",
        colour = "white",
        size = 2,
        linetype = "solid"
      ),
      panel.grid.major = ggplot2::element_line(
        size = 0.5,
        linetype = 'solid',
        colour = "#DFDFDF"
      ), 
      panel.grid.minor = ggplot2::element_line(
        size = 0.25,
        linetype = 'solid',
        colour = "#DFDFDF"
      ),
      plot.title = ggplot2::element_text(
        size = 13.5,
        face = "bold",
        hjust = 0.5
      ),
      axis.text = ggplot2::element_text(
        size = 10
      ),
      axis.title = ggplot2::element_text(
        face = "italic",
        size = 10
      ),
      # axis.ticks = element_blank(),
      axis.line.y.left = ggplot2::element_line(color = "black"),
      axis.line.x.bottom = ggplot2::element_line(color = "black")
    )
  return(p)
}